% ----------------------------------------------------------------------
% GNURBS Lab is a user-friendly and interactive toolbox which is developed
% for demonstrating the behaviour of GNURBS curves and how it compares to
% NURBS. This toolbox is developed by the extension of an opensource MATLAB
% Toolbox B-spline Lab with the following specifications:
%
% B-spline Lab is free software: you can redistribute it and/or modify it under the terms of the 
% GNU Lesser General Public License as published by the Free Software Foundation, either version 3 
% of the License, or (at your option) any later version.
% 
% B-spline Lab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
% even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
% Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Lesser General Public License along with B-spline Lab. 
% If not, see http://www.gnu.org/licenses/.

% -----------------------------------------------------------------------
%
% B-spline Lab
% P.J. Barendrecht
%
% -----------------------------------------------------------------------

function GNURBS_Lab(varargin)

if nargin == 0
  clear all
end

close all
clc

fprintf(['--------------------\n', ...
  'GNURBS Lab\nPress ''H'' for help.\n', ...
  '--------------------\n\n'])
global H H2 uc Degree Xi Points Weights status sl1 Xi_init str_w Gn1 h2 AXIS scrsz flag
%Gn1: a variable that controls if to show the general nurb plot or not
Gn1=0;
scrsz = get(groot,'ScreenSize');
flag = 1;
H = figure('OuterPosition', [scrsz(3)/6 scrsz(4)/6 scrsz(3)*4/6 scrsz(4)*5/6],'NumberTitle','off','Name','GNURBS Lab [Vertex]');
% daspect([0.5 1 1]); %pbaspect([1 1 1])
H2=0; %handle to basis function figure
%set(H, 'MenuBar','none');
set(H,'ToolBar','none');
set(H, 'Renderer','OpenGL');
hold on

set(H,'WindowButtonMotionFcn',@MouseMove);
set(H,'WindowButtonDownFcn',@MouseClick);
set(H,'WindowScrollWheelFcn',@MouseScroll);
set(H,'KeyPressFcn',@KeyPress )
grid on


hp = uipanel('parent',H,'Title','Main Panel','FontSize',12,...
              'Position',[0 0 .1 .95]);
%  uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Move Control Point',...
%     'Position', [0 0 100 20],'callback',@movepoint);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Basis Functions',...
    'Position', [0 25 100 20],'callback',@viewbasefuncs);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Hodograph',...
    'Position', [0 50 100 20],'callback',@viewhodograph);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Curve Information',...
    'Position', [0 75 100 20],'callback',@showinfo);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Show/Hide axis',...
    'Position', [0 100 100 20],'callback',@showaxis);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Show/Hide Grid',...
    'Position', [0 125 100 20],'callback',@showgrid);

%%%%%%%%%%%%%%%%%%%%%
uicontrol('parent',hp,'Style', 'text', 'String', 'Enter number of control points:',...
    'Position', [0 560 100 30]);
uicontrol('parent',hp,'Style', 'edit', 'backgroundcolor','white',...
    'Position', [5 540 100 20],'callback',@initialncp); %initial number of control points

uicontrol('parent',hp,'Style', 'text', 'String', 'Enter Degree:',...
    'Position', [0 520 100 20]);
uicontrol('parent',hp,'Style', 'edit', 'backgroundcolor','white',...
    'Position', [5 505 100 20],'callback',@changedeg,'ButtonDownFcn',@degfun);

uicontrol('parent',hp,'Style', 'text', 'String', 'Enter Weight:',...
    'Position', [0 480 100 20]);
uicontrol('parent',hp,'Style', 'edit', 'backgroundcolor','white',...
    'Position', [5 465 100 20],'callback',@Weightchange);

uicontrol('parent',hp,'Style', 'text', 'String', 'Evaluate At t:',...
    'Position', [0 435 100 20]);
uicontrol('parent',hp,'Style', 'edit', 'backgroundcolor','white',...
    'Position', [5 420 100 20],'callback',@parametereva);

uicontrol('parent',hp,'Style', 'text', 'String', 'Cont. Point Coords:',...
    'Position', [0 395 125 20]);
uicontrol('parent',hp,'Style', 'edit', 'backgroundcolor','white',...
    'Position', [5 380 100 20],'callback',@controlpointcoor);

uicontrol('parent',hp,'Style', 'text', 'String', 'Selection Mode:',...
    'Position', [0 345 125 20]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ub=uibuttongroup('parent',hp,'Units','Pixels','Position', [5 305 120 50]);
u0 = uicontrol('parent',ub,'Style','Radio','String','Vertex',...
    'position',[5 5 60 20],'HandleVisibility','off');
u1 = uicontrol('parent',ub,'Style','Radio','String','Edge',...
    'position',[5 25 60 20],'HandleVisibility','off');
set(ub,'SelectionChangeFcn',@EVtoggle);  %toggle between edge and vertice selection
%set(ub,'SelectedObject');

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Show/Hide knots',...
    'Position', [0 150 100 20],'callback',@showknots);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Reset Weights',...
    'Position', [0 175 100 20],'callback',@Resetweights);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Toggle Labels',...
    'Position', [0 200 100 20],'callback',@cpcoords);  %coordinate of control points

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'Reset Knot Vector',...
    'Position', [0 225 100 20],'callback',@ResetKnot);

uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'RESET GRAPH',...
    'Position', [0 250 100 20],'callback',@Resetgraph);
uicontrol('parent',hp,'Style', 'pushbutton', 'String', 'QUIT',...
    'Position', [0 275 100 20],'callback',@quit);

status=uicontrol('parent',H,'Style', 'edit', 'String', 'status',...
    'fontsize',12,'Units','normalized','Position', [.35 .9 .31 .1]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hp2 = uipanel('parent',H,'Title','Main Panel','FontSize',12,...
              'Position',[.9 0 .1 .95]);     
uc=uibuttongroup('parent',hp2,'Units','Pixels','Position', [5 300 120 65]);
uicontrol('parent',uc,'Style','Radio','String','xy',...
    'position',[5 5 60 20],'HandleVisibility','off');
uicontrol('parent',uc,'Style','Radio','String','xz',...
    'position',[5 25 60 20],'HandleVisibility','off');
uicontrol('parent',uc,'Style','Radio','String','yz',...
    'position',[5 45 60 20],'HandleVisibility','off');
set(uc,'SelectionChangeFcn',@projection);

uicontrol('parent',hp2,'Style', 'checkbox', 'String', 'rotate3d',...
    'Position', [5 370 100 20],'callback',@rot3d);

uicontrol('parent',hp2,'Style', 'checkbox', 'String', 'Eq.NURBS',...
    'Position', [5 270 100 20],'callback',@showGnurb);

ud=uibuttongroup('parent',hp2,'Units','Pixels','Position', [5 190 120 65]);
uicontrol('parent',ud,'Style','Radio','String','ISO',...
    'position',[5 5 60 20],'HandleVisibility','off');
uicontrol('parent',ud,'Style','Radio','String','WX',...
    'position',[5 25 60 20],'HandleVisibility','off');
uicontrol('parent',ud,'Style','Radio','String','WY',...
    'position',[5 45 60 20],'HandleVisibility','off');
set(ud,'SelectionChangeFcn',@weighttype);
str_w='ISO';

uicontrol('parent',hp2,'Style', 'text', 'String', 'changing knot vector',...
  'Position', [5 10 100 30]);
sl1=uicontrol('parent',hp2,'style','slider','string','knot','Position',...
    [50 50 20 120],'min',-5,'max',5,'callback',@changeknot);  %sl1:slider #1

Points = [];
Weights = [];
MinX = 1;
MaxX = 9;
MinY = 1;
MaxY = 9;
AXIS=[ MinX-1, MaxX+1, MinY-1, MaxY+1 ];
axis(AXIS )
%axis off

global SelectMode Grabbed Snap Labels Joints Index 

SelectMode = 1;
Grabbed = 0;
Snap = 0;
Labels = 1;
Joints = 1;
Index = [];

% SetupCurve()

% -----------------------------------------------------------------------

function SetupCurve()
global Basis Degree Xi Points Weights Weights_x Weights_y Index status Xi_init N
%%%%%%%%%%%%
% N=7;
M=N+Degree+1; %number of knots
mp=(M)-2*(Degree+1); %number of middle points in knot vector
%defining knot vector:in this vector using clamped condition first (p+1)
%knots are zero and last(p+1) knots are one, and middle knots are
%distributed equally //////////
Xi=zeros(1,M); %knot vector
for i=1:Degree+1
    Xi(1,i)=0;
    Xi(1,M-(i-1))=1;
end
for i=1:mp
    Xi(1,Degree+1+i)=i*(1-0)/(mp+1);
end

Xi_init=Xi;  %initial knot vector
%%%%%%%%%%%%%%%%%%
[~, Basis] = CoxdeBoor(Degree,Xi,Degree+1,length(Xi)-Degree);
NumberPoints = length(Xi) - Degree - 1;

if size(Points,1) < NumberPoints
  AddPoints = NumberPoints - size(Points,1);
%   disp(['Please add ', num2str(AddPoints), ' control points.'])

set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
    ['Please add ', num2str(AddPoints), ' control points.'],'foregroundcolor','black')

  set(gcf,'Pointer','FullCrossHair')
elseif size(Points,1) > NumberPoints
  DeletePoints = size(Points,1) - NumberPoints;
  Points = Points(1:end-DeletePoints,:);
  Index = [];
  Weights = Weights(1:end-DeletePoints);
  Weights_x = Weights_x(1:end-DeletePoints);
  Weights_y = Weights_y(1:end-DeletePoints);
%   disp([':: Deleted ', num2str(DeletePoints), ' control points.'])
set(status,'max',1,'Units','normalized','position',[.35 .9 .31 .1],'string',... 
    [':: Deleted ', num2str(DeletePoints), ' control points.'],'foregroundcolor','black')
  UpdateCurve()
else
  UpdateCurve()
end
function SetupCurve_chnageknot()
global Basis Degree Xi Points Weights Weights_x Weights_y Index status Xi_init N  
%%%%%%%%%%%%
% N=7;
%%%%%%%%%%%%%%%%%%
[~, Basis] = CoxdeBoor(Degree,Xi,Degree+1,length(Xi)-Degree);
NumberPoints = length(Xi) - Degree - 1;

if size(Points,1) < NumberPoints
  AddPoints = NumberPoints - size(Points,1);
%   disp(['Please add ', num2str(AddPoints), ' control points.'])

set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
    ['Please add ', num2str(AddPoints), ' control points.'],'foregroundcolor','black')

  set(gcf,'Pointer','FullCrossHair')
elseif size(Points,1) > NumberPoints
  DeletePoints = size(Points,1) - NumberPoints;
  Points = Points(1:end-DeletePoints,:);
  Index = [];
  Weights = Weights(1:end-DeletePoints);
  Weights_x = Weights_x(1:end-DeletePoints);
  Weights_y = Weights_y(1:end-DeletePoints);
%   disp([':: Deleted ', num2str(DeletePoints), ' control points.'])
set(status,'max',1,'Units','normalized','position',[.35 .9 .31 .1],'string',... 
    [':: Deleted ', num2str(DeletePoints), ' control points.'],'foregroundcolor','black')
  UpdateCurve()
else
  UpdateCurve()
end

% -----------------------------------------------------------------------

function UpdateCurve()
global Basis Degree Xi Points Weights Weights_x Weights_y Index Labels Joints SelectMode str_w Gn1 H2 AXIS H scrsz
cla

% Control polygon
plot( Points(:,1), Points(:,2), '-', 'Color', .6*[1 1 1], 'LineWidth', 1 );

% Control points
plot( Points(:,1), Points(:,2), 'ko', 'MarkerSize', 12, 'MarkerFaceColor', [0 0 0] );

switch Labels
  case 1
    % Control point labels
    for P = 1:length(Points)
      text( Points(P,1), Points(P,2), ['    P_{', num2str(P), '}'], 'FontSize', 12 )
    end
  case 2
    % Weights
    for P = 1:length(Points)
      text( Points(P,1), Points(P,2), ['    <', num2str(Weights(P)), '>'], 'FontSize', 12 )
    end
  case 3
    % Blossom labels
    for P = 1:length(Points)
      Label = strtrim( sprintf('%.3g  ', Xi(P+1:P+Degree)) );
      text( Points(P,1), Points(P,2), ['    (', Label, ')' ], 'FontSize', 12 )
    end
  case 4
    % Knot spans/intervals
    if mod(Degree,2) == 0
      % Knot spans associated with control points
      for P = 1:length(Points)
        text( Points(P,1), Points(P,2), ['    [', num2str( Xi(round(Degree/2)+1+P)-Xi(round(Degree/2)+P) ), ']'], 'FontSize', 12 )
      end
    else
      % Knot spans associated with edges
      for E = 1:length(Points)-1
        text( (Points(E,1)+Points(E+1,1))/2, (Points(E,2)+Points(E+1,2))/2, ['[', num2str( Xi(round(Degree/2)+1+E)-Xi(round(Degree/2)+E) ), ']'], 'BackGroundColor', .8*[1 1 1], 'FontSize', 12 )
      end
    end
    
    case 5
        %coordinate of control points
        for P=1:length(Points)
         text( Points(P,1), Points(P,2),...
             ['    (',strtrim(sprintf('%.2g  ', Points(P,1))),',',strtrim(sprintf('%.2g  ', Points(P,2))),')'], 'FontSize', 12 )
        end
end
if strcmp(str_w,'WX')
Denominator_x = Weights_x * Basis;
Denominator_y = Weights_y * Basis;
X = (Weights_x.*Points(:,1)' * Basis) ./ Denominator_x;
Y = (Weights_y.*Points(:,2)' * Basis) ./ Denominator_y;
elseif strcmp(str_w,'WY')
Denominator_y = Weights_y * Basis;
Denominator_x = Weights_x * Basis;
X = (Weights_x.*Points(:,1)' * Basis) ./ Denominator_x;
Y = (Weights_y.*Points(:,2)' * Basis) ./ Denominator_y;
elseif strcmp(str_w,'ISO')
Denominator = Weights * Basis;
X = (Weights.*Points(:,1)' * Basis) ./ Denominator;
Y = (Weights.*Points(:,2)' * Basis) ./ Denominator;
end

plot( X, Y, 'Color', [255 128 0]/255, 'LineWidth', 3 );

if Joints
  Knots = unique(Xi((Degree+1):(end-Degree)));
  Pts = [];
  for Val = Knots
    Joint = findknotval(Val);
    plot( Joint(1), Joint(2), 's', 'Color', [255 128 0]/255, 'MarkerSize', 10, 'MarkerFaceColor', [255 128 0]/255 )
    text( Joint(1), Joint(2)-.3, ['t = ', strtrim(sprintf('%.3g  ', Val))], 'FontSize', 12 )
  end
end

if SelectMode == 1
  plot( Points(Index,1), Points(Index,2), 'o', 'MarkerSize', 16, 'Color', .9*[1 1 1], 'LineWidth', 3 );
else
  plot( [Points(Index,1) Points(Index+1,1)], [Points(Index,2) Points(Index+1,2)], 'Color', .9*[1 1 1], 'LineWidth', 3 );
end
if Gn1==1  %the ploting equivalent nurb is active
    if strcmp(str_w,'WX') || strcmp(str_w,'WY')
[PX,PY]=Gnurbs1(Degree,Points',[Weights_x;Weights_y]);
plot(PX,PY,'Color',[0.5 0.5 1],'marker','o','MarkerSize',8,'markerfacecolor',[0.5 0.3 1],'MarkerEdgeColor','k','LineWidth',1.5);
    elseif strcmp(str_w,'ISO')
        [PX,PY]=Gnurbs1(Degree,Points',[Weights;Weights]);
        plot(PX,PY,'Color',[0.5 0.5 1],'marker','o','MarkerSize',8,'markerfacecolor',[0.5 0.3 1],'MarkerEdgeColor','k','LineWidth',1.5);

    end     
end
if double(H2) && ishandle(H2) 
    PlotBasis();
    axis(AXIS)
%     set(H,'position',[100 300 1300 400])
%     set(H,'position',[scrsz(3)/6 scrsz(4)/6 scrsz(3)/1.5 scrsz(4)/1.3])
% else
% set(H,'position',[100 100 1300 600])    
end

% -----------------------------------------------------------------------

function MouseMove(~,~)
global Grabbed MoveAlong Snap Points Index  

if Grabbed
  CurrentPoint = get(gca,'CurrentPoint');
  Pt = CurrentPoint(1,1:2);
  
  if Snap
    % Snap to grid
    Steps = 4;
    Pt = round(Steps*Pt) / Steps;
  end
  
  switch MoveAlong
    case 0
      Points(Index,:) = Pt;
    case 1
      Points(Index,1) = Pt(1);
    case 2
      Points(Index,2) = Pt(2);
  end
  
  UpdateCurve()
end

% -----------------------------------------------------------------------

function MouseClick(~,~)
global SelectMode MoveAlong Grabbed Snap Points Weights Weights_x Weights_y  Xi Degree Index status Rclick sl1 H2 H flag scrsz

% Left click
if isobject(H2) && ~ishandle(H2) && flag~=0
%     hfig =  findobj('type','figure');
%     nfig = length(hfig);
%     if nfig == 1 && H2.OuterPosition == scrsz(4)/3.3 && (double(H2)==0 || ~ishandle(H2))
        set(H,'OuterPosition', [scrsz(3)/6 scrsz(4)/6 scrsz(3)*4/6 scrsz(4)*5/6])
        flag = 0;
%     end
end

if strcmpi(get(gcf,'SelectionType'), 'Normal')
Rclick=0;  
  if Grabbed
    Grabbed = 0;
    Index = [];
    disp(':: Releasing control point')
set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Releasing control point','foregroundcolor','black')
    UpdateCurve();
  else
    if strcmpi( get(gcf,'Pointer'),'FullCrossHair')
      
      CurrentPoint = get(gca,'CurrentPoint');
      Pt = CurrentPoint(1,1:2);
      
      if Snap
        % Snap to grid
        Steps = 4;
        Pt = round(Steps*Pt) / Steps;
      end
      
      Points(end+1,:) = Pt;
      Weights(end+1) = 1;
      Weights_x(end+1)=1;
      Weights_y(end+1)=1;
      plot( Points(end,1), Points(end,2), 'ko', 'MarkerSize', 12, 'MarkerFaceColor', [0 0 0] );
      disp(':: Added a new control point')
set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Added a new control point','foregroundcolor','black')
      
      if size(Points,1) == (length(Xi)-Degree-1)
        set(gcf,'Pointer','Cross')
        UpdateCurve()
      end
      
    end
  end
  
  % Right click
elseif strcmpi(get(gcf,'SelectionType'), 'Alt')
Rclick=1;




%     if (SelectMode == 1 && length(Index))
%       disp(':: Grabbing control point')
%       Grabbed = 1;
%       MoveAlong = 0;
%     end
  
  if SelectMode == 1
    disp(':: Selecting control point')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Selecting control point','foregroundcolor','black')
    Index = FindClosest();
    UpdateCurve()
    
  elseif SelectMode == 2
      set(sl1,'value',0);
    disp(':: Selecting edge')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Selecting edge','foregroundcolor','black')
    Index = FindClosest();
    UpdateCurve()
  end
  
elseif strcmpi(get(gcf,'SelectionType'), 'open') 
Grabbed = 1;
MoveAlong = 0;    
UpdateCurve() 
end



% -----------------------------------------------------------------------

function MouseScroll(~,Event)
global SelectMode Weights Weights_x Weights_y Index status str_w

if SelectMode == 1
  Scale = 10;
  Value = Event.VerticalScrollCount / Scale;
  if strcmp(str_w,'ISO')
  Weights(Index) = Weights(Index) + Value;
  set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
      ['scroll to change weight: ',num2str(Weights(Index))],'foregroundcolor','black')
  if Weights(Index) < 0
    warning('Weight value negative!')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
        ['Weight value negative!',num2str(Weights(Index))],'foregroundcolor','red')
  end

  elseif strcmp(str_w,'WX')
  Weights_x(Index) = Weights_x(Index) + Value;
  set(status,'max',2,'Units','normalized','Position', [.35 .88 .31 .1],'string',...
      {['scroll to change weight_x: ',num2str(Weights_x(Index))],...
      ['weight_y:  ',num2str(Weights_y(Index))]},'foregroundcolor','black')
  if Weights_x(Index) < 0
    warning('Weight_x value negative!')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
        ['Weight_x value negative!',num2str(Weights_x(Index))],'foregroundcolor','red')
  end
  
  elseif strcmp(str_w,'WY')
  Weights_y(Index) = Weights_y(Index) + Value;
  set(status,'max',2,'Units','normalized','Position', [.35 .88 .31 .1],'string',...
      {['scroll to change weight_y: ',num2str(Weights_y(Index))],...
      ['weight_x: ',num2str(Weights_x(Index))]},'foregroundcolor','black')
  if Weights_y(Index) < 0
    warning('Weight_y value negative!')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
        ['Weight_y value negative!',num2str(Weights_y(Index))],'foregroundcolor','red')
  end
  end
  
  UpdateCurve()
end

% -----------------------------------------------------------------------

function KeyPress(~,Event)
global Xi Degree Labels Joints Points Weights Index SelectMode Grabbed Snap MoveAlong 

switch Event.Character
    case 27
        Grabbed=0;
        UpdateCurve()
  case {'a', 'A'}
    % Toggle axis visibility
    if strcmpi(get(gca, 'Visible'), 'On')
      axis off;
    else
      axis on;
    end
  case {'b', 'B'}
    % Plot the basis
    PlotBasis();
  case {'c', 'C'}
    % Show Bezier extraction matrices [TODO]
  case {'d', 'D'}
    % Change the degree
    Deg = input('Degree (e.g. quadratic = 2): ');
    if length(Xi) < (2 + 2*Deg)
      warning('Knot vector does not satisfy minimal length for this degree!')
    else
      Degree = Deg;
      SetupCurve()
    end
  case {'e', 'E'}
    % Set select mode to Edges
    SelectMode = 2;
    Index = [];
    set(gcf,'Name','GNURBS Lab [Edge]');
    disp(':: Select Mode [Edge]')
    UpdateCurve();
  case {'f', 'F'}
    % Print graphical overview of the support of the basis functions
    PrintSupports()
  case {'g', 'G'}
    % Grab control point
    if (SelectMode == 1 && length(Index))
      disp(':: Grabbing control point')
      Grabbed = 1;
      MoveAlong = 0;
    end
  case {'h', 'H'}
    % Plot hodograph of the curve
    Hodograph()
  case {'i', 'I'}
    % Display information about the curve
    Knts = strtrim(sprintf('%.4g  ', Xi));
    Wts = strtrim(sprintf('%.4g  ', Weights));
    fprintf(['\nKnotvector: [', Knts, ']\n', ...
      'Degree: ', num2str(Degree), '\n', ...
      'Number of control points: ', num2str(size(Points,1)), '\n', ...
      'Weights: ', Wts, '\n\n'])
  case {'j', 'J'}
    % Toggle visibility of the joints (images of the knots)
    Joints = ~Joints;
    UpdateCurve()
  case {'k', 'K'}
    % Change the knotvector
    Xi = input('Knotvector (between [ and ]): ');
    SetupCurve()
  case {'l', 'L'}
    % Cycle through label types (0 no labels, 1 control point labels, 2 weights, 3 blossom labels, 4 knot spans)
    Labels = mod(Labels+1,5);
    switch Labels
      case 1
        disp(':: Displaying control point labels')
      case 2
        disp(':: Displaying weights')
      case 3
        disp(':: Displaying blossom (polar form) labels')
      case 4
        disp(':: Displaying knot spans (intervals)')
    end
    UpdateCurve()
  case {'m', 'M'}
    % Global midpoint knot refinement (Lane-Riesenfeld) [TODO]
  case {'n', 'N'}
    % Placeholder
  case {'o', 'O'}
    % Order elevation [TODO]
  case {'p', 'P'}
    % Set exact position of selected control point
    if SelectMode == 1
      Points(Index,:) = input('Position (between [ and ]): ');
      UpdateCurve()
    end
  case {'q', 'Q'}
    % Quit
    close all
    clear all
    clc
  case {'r', 'R'}
    % Local refinement (knot insertion, Boehm) [TODO]
  case {'s', 'S'}
    % Save the curve in SVG format using plot2svg
    % See http://www.mathworks.nl/matlabcentral/fileexchange/7401-scalable-vector-graphics-svg-export-of-figures
    % Also http://www.mathworks.nl/matlabcentral/fileexchange/23629-exportfig
    plot2svg
  case {'t', 'T'}
    % Evaluate curve at parameter value t (scaffolding approach)
    Scaffold = strcmpi(get(gcf,'CurrentModifier'), 'Shift');
    Val = input('Evaluate curve at parameter value: ');
    if Val < Xi(Degree+1) || Val >= Xi(end-Degree)
      warning('Parameter value out of range!')
    end
    Pt = ExactEval(Val,Scaffold)
    plot(Pt(1), Pt(2), 'rx', 'MarkerSize', 12, 'LineWidth', 2)
    text(Pt(1), Pt(2)-.3, ['t = ', num2str(Val)], 'Color', [1 0 0], 'FontSize', 12 )
  case {'u', 'U'}
    % Reset weights to unity
    Weights = ones(1,length(Weights));
    UpdateCurve()
  case {'v', 'V'}
    % Set select mode to Vertices
    SelectMode = 1;
    Index = [];
    set(gcf,'Name','GNURBS Lab [Vertex]');
    disp(':: Select Mode [Vertex]')
    UpdateCurve();
  case {'w', 'W'}
    % Set exact weight of selected control point
    if SelectMode == 1
      Weights(Index) = input('Weight: ');
      UpdateCurve();
    end
  case {'x', 'X'}
    % Toggle constrained movement in X direction
    if Grabbed
      MoveAlong = ~MoveAlong;
    end
  case {'y', 'Y'}
    % Toggle constrained movement in Y direction
    if Grabbed
      MoveAlong = 2*(~MoveAlong);
    end
  case {'z', 'Z'}
    % Show curve in homogeneous space (along with perspective projection) [TODO]
    
  case {'#'}
    % Toggle snap to "grid"
    Snap = ~Snap;
  case {'+'}
    % Increase knot-span associated with currently selected vertex or edge
    if (mod(Degree,2) == 0 && SelectMode == 1 && length(Index)) || (mod(Degree,2) == 1 && SelectMode == 2 && length(Index))
      Xi = [Xi(1:round(Degree/2)+Index) Xi(round(Degree/2)+1+Index:end) + .05];
      SetupCurve()
    end
  case {'-'}
    % Decrease knot-span
    if (mod(Degree,2) == 0 && SelectMode == 1 && length(Index)) || (mod(Degree,2) == 1 && SelectMode == 2 && length(Index))
      if Xi(round(Degree/2)+Index) <= Xi(round(Degree/2)+1+Index) - .05
        Xi = [Xi(1:round(Degree/2)+Index) Xi(round(Degree/2)+1+Index:end) - .05];
        SetupCurve()
      end
    end
  case {'?'}
    % Show help [TODO]
    disp('Help')
end

% -----------------------------------------------------------------------




function movepoint(hObject, eventdata, handles)
global Index SelectMode Grabbed  MoveAlong status Rclick
if Rclick==0
set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
          'Please first select a control point by Right Click','foregroundcolor','black');
end
if (SelectMode == 1 && length(Index))
      disp(':: Grabbing control point')
      set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
          ':: moving control point','foregroundcolor','black');
      Grabbed = 1;
      MoveAlong = 0;
end

function viewbasefuncs(hObject, eventdata, handles)
global H scrsz
set(H,'OuterPosition',[scrsz(3)/6 scrsz(4)/3 scrsz(3)*4/6 scrsz(4)*4/6]) 
PlotBasis();

function viewhodograph(hObject, eventdata, handles)
Hodograph();

function showinfo(hObject, eventdata, handles)
global Xi Degree Points Weights status Weights_x Weights_y
 Knts = strtrim(sprintf('%.4g  ', Xi));
    Wts = strtrim(sprintf('%.4g  ', Weights));
    Wtsx = strtrim(sprintf('%.4g  ', Weights_x));
    Wtsy = strtrim(sprintf('%.4g  ', Weights_y));
    fprintf(['\nKnotvector: [', Knts, ']\n', ...
      'Degree: ', num2str(Degree), '\n', ...
      'Number of control points: ', num2str(size(Points,1)), '\n', ...
      'Weights: ', Wts, '\n\n'])
set(status,'Units','normalized','Position', [.35 .75 .31 .17])
set(status,'max',6)
set(status,'string',{['Knotvector: [', Knts, ']'], ...
      ['Degree: ', num2str(Degree)], ...
      ['Number of control points: ', num2str(size(Points,1))], ...
      ['Weights(ISO): ', Wts],['Weights(WX): ', Wtsx],['Weights(WY): ', Wtsy]},'foregroundcolor','black')
 
 function showaxis(hObject, eventdata, handles)
if strcmpi(get(gca, 'Visible'), 'On')
      axis off;
    else
      axis on;
end 

 function showgrid(hObject, eventdata, handles)
if strcmpi(get(gca, 'Xgrid'), 'On')
      grid off;
    else
      grid on;
end 


function changedeg(hObject, eventdata, handles)
global Xi Degree status
set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
    'Enter the value for new Degree','foregroundcolor','black')
Deg = str2double(get(hObject,'string'));
    if length(Xi) < (2 + 2*Deg)&& ~isempty(Xi)
      warning('Knot vector does not satisfy minimal length for this degree!')
      set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],...
          'string','Knot vector does not satisfy minimal length for this degree!','foregroundcolor','black')
    else
      Degree = Deg;
      SetupCurve()
    end 
    
function showknots(hObject, eventdata, handles)
    global Joints
   Joints = ~Joints;
   UpdateCurve()
   
   
function Resetweights(hObject, eventdata, handles)
  global Weights Weights_x Weights_y SelectMode status
   Weights = ones(1,length(Weights));
   Weights_x = ones(1,length(Weights_x));
   Weights_y = ones(1,length(Weights_y));
    UpdateCurve()
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1] ,'string','Weights Reset to 1','foregroundcolor','black')
    
    
function Weightchange(hObject, eventdata, handles)
global SelectMode Weights Weights_x Weights_y Index status Rclick str_w
W=str2double(get(hObject,'string'));
if Rclick==0
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
        'Please first select a control point by Right Click','foregroundcolor','black')    
end
if SelectMode == 1 
    if strcmp(str_w,'ISO') 
      Weights(Index) = W;
    elseif strcmp(str_w,'WX')
      Weights_x(Index) = W;
    elseif strcmp(str_w,'WY')
        Weights_y(Index) = W;
    end
      UpdateCurve();
end


function EVtoggle(source,eventdata)
global SelectMode Index status STR
STR=get(eventdata.NewValue,'String');

 if strcmp(STR,'Edge')
 SelectMode = 2;
    Index = [];
    set(gcf,'Name','GNURBS Lab [Edge]');
    disp(':: Select Mode [Edge]')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Select Mode [Edge]','foregroundcolor','black')
    UpdateCurve();
    
 elseif strcmp(STR,'Vertex')
 SelectMode = 1;
    Index = [];
    set(gcf,'Name','GNURBS Lab [Vertex]');
    disp(':: Select Mode [Vertex]')
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',':: Select Mode [Vertex]','foregroundcolor','black')
    UpdateCurve();
 end
 
function parametereva(hObject, eventdata, handles)
global  Xi Degree  status
% Evaluate curve at parameter value t (scaffolding approach)
     Val = str2double(get(hObject,'string'))       
 
    if Val < Xi(Degree+1) || Val >= Xi(end-Degree)
      warning('Parameter value out of range!')
      set(status,'max',2,'Units','normalized','Position', [.35 .84 .31 .1],'string','Warning: Parameter value out of range!',...
          'foregroundcolor','black')
    else
        
    Pt = findknotval(Val);
    Pts=strtrim(sprintf('%.4g  ', Pt));
    plot(Pt(1), Pt(2), 'rx', 'MarkerSize', 12, 'LineWidth', 2)
    text(Pt(1), Pt(2)-.3, ['t = ', num2str(Val)], 'Color', [1 0 0], 'FontSize', 12 )
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',['the xy position is:  ',Pts],'foregroundcolor','black')    
    end
    
    
function controlpointcoor(hObject, eventdata, handles)
global SelectMode Index Points status Rclick
if Rclick==0
    set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string',...
        'Please first select a control point by Right Click','foregroundcolor','black')    
end
str=get(hObject,'string');
S=size(str,2);
i=1;
while ~strcmpi(str(i),',')
    i=i+1;
end
    
x=str2double(str(1:i-1));
y=str2double(str(i+1:S));
% Set exact position of selected control point
    if SelectMode == 1
      Points(Index,1) =x; 
      Points(Index,2) =y; 
      UpdateCurve()
    end
    
    
function cpcoords(hObject, eventdata, handles)
global Labels status
% Cycle through label types (0 no labels, 1 control point labels, 2
% weights, 3 blossom labels, 4 knot spans)
Labels=Labels+1;
if Labels==6
    Labels=0;
end
switch Labels
    case 0
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','No Labels ','foregroundcolor','black')
    case 1
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Control Point Labels ','foregroundcolor','black')
    case 2
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Control Point Weights ','foregroundcolor','black')
    case 3
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Blossom Labels ','foregroundcolor','black')
    case 4
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Knot Spans ','foregroundcolor','black')
    case 5
        set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Control Point Coordinates ','foregroundcolor','black')
end
UpdateCurve();

function changeknot(hObject, eventdata, handles)
global Basis Degree Xi Points Weights Index Labels Joints SelectMode status Rclick
if SelectMode==1
 set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Please first select Edge from the left panel ','foregroundcolor','black')
elseif (SelectMode==2 && Rclick==0)
 set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Please Right Click to select an edge ','foregroundcolor','black')
else
  set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','::Changing Knot Vector  ','foregroundcolor','black')
end    
temp=get(hObject,'value');
% if temp~=0
%     set(hObject,'value',0)
% end
    if (mod(Degree,2) == 0 && SelectMode == 1 && length(Index)) || (mod(Degree,2) == 1 && SelectMode == 2 && length(Index))
      Xi = [Xi(1:round(Degree/2)+Index) Xi(round(Degree/2)+1+Index:end) + sign(temp)*.5];
      SetupCurve_chnageknot()
    end
    
function ResetKnot(hObject, eventdata, handles)
global Basis Degree Xi Points Weights Index Labels Joints SelectMode status sl1 Xi_init
set(sl1,'value',0);
Xi = Xi_init;
SetupCurve();

function Resetgraph(hObject, eventdata, handles)
GNURBS_Lab

function degfun(hObject, eventdata, handles)
global status
  set(status,'max',1,'Units','normalized','Position', [.35 .9 .31 .1],'string','Enter the value for new Degree','foregroundcolor','black')

function quit(hObject, eventdata, handles)
    close all
    clear all
    
    
function changecamera(hObject, eventdata, handles)
global H
temp=get(hObject,'value');
%view(3);
%campos([temp,temp,temp])
%camroll(temp)
 %camorbit(0,temp,'coordsys',[1,0,0])
 %rotate3d
%camup([temp,temp,temp])
% rotate(gca,[1,1,0],temp,[0,0,0])
%[az,el] = view
% temp
%   view([temp,0,1])
%  camroll(temp)
rotate3d on

function rot3d(hObject, eventdata, handles)
if (get(hObject,'Value') == get(hObject,'Max'))
   rotate3d on
else
rotate3d off 
end
%     str=get(uc.handles.NewValue,'String');
% if strcmp(str,'xy')
%     view([0,0,1])
% elseif strcmp(str,'xz')
% view([0,1,0])
% elseif strcmp(str,'yz')
% view([1,0,0]) 
% end


function projection(hObject, eventdata, handles)
str=get(eventdata.NewValue,'String');
if strcmp(str,'xy')
    view([0,0,1])
elseif strcmp(str,'xz')
view([0,1,0])
elseif strcmp(str,'yz')
view([1,0,0]) 
end

function weighttype(hObject, eventdata, handles)
global str_w H2
str_w=get(eventdata.NewValue,'String');
if double(H2)~=0 && ishandle(H2)
PlotBasis()
end

function showGnurb(hObject, eventdata, handles)
global Degree  Points Weights Weights_x Weights_y Gn1 
if (get(hObject,'Value') == get(hObject,'Max'))
    Gn1=1;
    [X,Y]=Gnurbs1(Degree,Points',[Weights_x;Weights_y]);
    plot(X,Y,'Color',[0.5 0.5 1],'marker','o','MarkerSize',8,'markerfacecolor',[0.5 0.3 1],'MarkerEdgeColor','k','LineWidth',1.5);
else
    Gn1=0;
    UpdateCurve();
end

function initialncp(hObject, eventdata, handles)
global N Degree
N=str2double(get(hObject,'string'));
if ~isempty(Degree)
    SetupCurve();
end





    
    


  
  
function Pt = ExactEval(Val, Scaffolding)
global Xi Degree Points

% Using de Boor's algorithm for exact evaluation (generalised de Casteljau)
% This is for B-splines, not for NURBS. Use perspective projection? [TODO]

% Determine relevant segment
Upper = length(Xi) - Degree;

l = 1;
while Val >= Xi(l)
  l = l+1;
  % Check
  if l == Upper
    break
  end
end
l = l-1;

CurrentPoints = Points((l-Degree):l,:);

for k = 1:Degree
  NewPoints = zeros(size(CurrentPoints,1)-1,2);
  
  for n = 1:size(NewPoints,1)
    Alpha = (Val-Xi(l-Degree+k+(n-1)))/(Xi(l+n)-Xi(l-Degree+k+(n-1)));
    NewPoints(n,:) = (1-Alpha)*CurrentPoints(n,:) + Alpha*CurrentPoints(n+1,:);
  end
  
  if Scaffolding
    plot( NewPoints(:,1), NewPoints(:,2), 'x-', 'MarkerSize', 12, 'Color', .4*[1 1 1], 'LineWidth', 1);
  end
  
  CurrentPoints = NewPoints;
  
end

Pt = CurrentPoints;

% -----------------------------------------------------------------------

function Index = FindClosest()
global Points SelectMode;

CurrentPoint = get(gca,'CurrentPoint');
Pt = CurrentPoint(1,1:2);

if SelectMode == 1
  
  Distances = zeros(1,length(Points));
  
  for P = 1:length(Distances)
    Distances(P) = norm( Points(P,:)-Pt, 2);
  end
  
else
  
  Distances = zeros(1,length(Points)-1);
  Px = Pt(1);
  Py = Pt(2);
  
  for E = 1:length(Distances)
    Ax = Points(E,1);
    Ay = Points(E,2);
    Bx = Points(E+1,1);
    By = Points(E+1,2);
    % Difference
    Dx = Bx - Ax;
    Dy = By - Ay;
    Val = ((Px-Ax)*Dx + (Py-Ay)*Dy)/(Dx^2 + Dy^2);
    
    if Val < 0
      Val = 0;
    elseif Val > 1
      Val = 1;
    end
    
    Distances(E) = norm( (1-Val)*Points(E,:)+Val*Points(E+1,:)-Pt, 2);
  end
  
end

[~,Index] = min( Distances );

% -----------------------------------------------------------------------

function Hodograph()
global Points Xi Degree

% Consider weights [TODO]

NewPts = zeros(length(Points)-1,2);

for Q = 1:length(NewPts)
  % Do not use first and last knot
  NewPts(Q,:) = (Degree/(Xi(Q+1+Degree)-Xi(Q+1)))*(Points(Q+1,:)-Points(Q,:));
end

H = figure('NumberTitle','off','Name','Hodograph');
set(H, 'MenuBar','none');
set(H,'ToolBar','none');
set(H, 'Renderer','OpenGL');
hold on
set(H,'KeyPressFcn',@HodoKeyPress )

% Control polygon
plot( NewPts(:,1), NewPts(:,2), '-', 'Color', .6*[1 1 1], 'LineWidth', 1 );

% Control points
plot( NewPts(:,1), NewPts(:,2), 'ko', 'MarkerSize', 12, 'MarkerFaceColor', [0 0 0] );

[~, NewBasis] = CoxdeBoor(Degree-1,Xi(2:end-1),Degree,length(Xi)-Degree-1);

X = NewPts(:,1)' * NewBasis;
Y = NewPts(:,2)' * NewBasis;

plot( X, Y, 'Color', [68 170 0]/255, 'LineWidth', 3 );
plot( 0,0, '+', 'MarkerSize', 14, 'LineWidth', 2, 'Color', [68 170 0]/255 );

for Q = 1:length(NewPts)
  text( NewPts(Q,1), NewPts(Q,2), ['   Q_{', num2str(Q), '}'], 'FontSize', 12 )
end

axis tight
axis off
axis equal

% -----------------------------------------------------------------------

function HodoKeyPress(~,Event)
switch Event.Character
  case {'q', 'Q'}
    % Quit
    close
  case {'s', 'S'}
    % Save the curve in SVG format using plot2svg
    plot2svg
end

% -----------------------------------------------------------------------

function PrintSupports()
global Xi Degree

Spaces = 0;

for B = 1:length(Xi)-Degree-1
  Support = Xi(B:B+Degree+1);
  fprintf('\n')
  fprintf(repmat(' ',1,Spaces))
  fprintf('|')
  PrevKnot = Support(1);
  
  for K = 2:length(Support)
    CurrentKnot = Support(K);
    if CurrentKnot == PrevKnot
      fprintf('|')
    else
      fprintf('-----|')
    end
    PrevKnot = CurrentKnot;
  end
  
  if Xi(B+1) == Xi(B)
    Spaces = Spaces + 1;
  else
    Spaces = Spaces + 6;
  end
end

fprintf('\n\n')

% -----------------------------------------------------------------------

function PlotBasis()
global H2 Xi Degree Points Weights Weights_x Weights_y h2 str_w scrsz flag
flag = 1;
if strcmpi(get(gcf,'CurrentModifier'), 'Shift')
  From = 1;
  To = length(Xi);
else
  From = Degree+1;
  To = length(Xi)-Degree;
end

[Evals, Basis] = CoxdeBoor(Degree,Xi,From,To);

% Colours in hexadecimal format
Colours = ['AA0088'; 'D40000'; 'FF8000'; 'FFCC00'; '44AA00'; '2CA089'; '3771C8'];

for B = 1:length(Points)
  k = mod(B,length(Colours))+1;
  Col = hex2dec([Colours(k,1:2); Colours(k,3:4); Colours(k,5:6)])'/255;
  plot(Points(B,1), Points(B,2), 'o', 'MarkerSize', 16, 'Color', Col, 'LineWidth', 3 );
end

% Estimation for window dimensions
Width = 1000; %Width = scrsz(3)/1.6;
% Height = Width / (length(Points) - Degree + 1) ;
Height=200; %Height = scrsz(4)/4;
if double(H2)==0 || ~ishandle(H2)
H2 = figure('NumberTitle','off','Name','Basis');
h2=gca(H2);
set(H2, 'MenuBar','none');
set(H2,'ToolBar','none');
set(H2,'OuterPosition',[scrsz(3)/6 scrsz(4)/24 scrsz(3)*4/6 scrsz(4)/3.3]);
set(H2, 'Renderer','OpenGL');
hold on
set(H2,'KeyPressFcn',@HodoKeyPress )
end
cla(h2)

if strcmp(str_w,'ISO')
    Denominator = Weights * Basis;
    for B = 1:length(Points)
        k = mod(B,length(Colours))+1;
        Col = hex2dec([Colours(k,1:2); Colours(k,3:4); Colours(k,5:6)])'/255;
        %plot(Evals, Basis(B,:), 'Color', Col, 'LineWidth', 2)
        plot(h2,Evals, Weights(B)*Basis(B,:) ./ Denominator, 'Color', Col, 'LineWidth', 2)
    end
elseif strcmp(str_w,'WX')
    Denominator = Weights_x * Basis;
        for B = 1:length(Points)
        k = mod(B,length(Colours))+1;
        Col = hex2dec([Colours(k,1:2); Colours(k,3:4); Colours(k,5:6)])'/255;
        %plot(Evals, Basis(B,:), 'Color', Col, 'LineWidth', 2)
        plot(h2,Evals, Weights_x(B)*Basis(B,:) ./ Denominator, 'Color', Col, 'LineWidth', 2)
        end
elseif strcmp(str_w,'WY')
    Denominator = Weights_y * Basis;
        for B = 1:length(Points)
        k = mod(B,length(Colours))+1;
        Col = hex2dec([Colours(k,1:2); Colours(k,3:4); Colours(k,5:6)])'/255;
        %plot(Evals, Basis(B,:), 'Color', Col, 'LineWidth', 2)
        plot(h2,Evals, Weights_y(B)*Basis(B,:) ./ Denominator, 'Color', Col, 'LineWidth', 2)
        end
end
grid on
% axis tight

% -----------------------------------------------------------------------

function Joint=findknotval(u)
global Degree Points Weights Weights_x Weights_y Xi str_w 
xN=Points(:,1)';
yN=Points(:,2)';
N=size(xN,2);
p=Degree;
if strcmp(str_w,'WX')|| strcmp(str_w,'WY')
Wx=Weights_x;
Wy=Weights_y;
elseif strcmp(str_w,'ISO')
Wx=Weights;
Wy=Weights;
end
% Wx=[1,1,2,1,5,10];
% Wy=[1,1,2,1,1,2];
% Wz=[1,1,2,1,2,2];
M=N+p+1;   %(number of knots)(m+1)
mp=(M)-2*(p+1); %number of middle points in knot vector
%defining knot vector:in this vector using clamped condition first (p+1)
%knots are zero and last(p+1) knots are one, and middle knots are
%distributed equally //////////
U=Xi;  %knotvector

%evaluating basis functions for different values of parameter u
 NN=zeros(M-1,p+1);  %matrix of bezier functions
%defining zero order basis functions(N0,0 , N1,0 , N2,0 , ...) which is the
%first column of NN matrix ////
for i=1:M-1;
    NN(i,1)=1*(stepp(u,U(i))-stepp(u,U(i+1)));
end
%/////////////////
   
    for j=1:p
        for i=0:(M-2)-j
        A=(u-U(i+1))/(U(i+j+1)-U(i+1));
        if isnan(A) || isinf(A)
            A=0;
        end
        B=(U(i+j+1+1)-u)/(U(i+j+1+1)-U(i+1+1));
        if isnan(B)|| isinf(B)
            B=0;
        end
        NN(i+1,j+1)=A*NN(i+1,j-1+1)+B*NN(i+1+1,j-1+1);

        end
    end
    
    %calculating rational basis functions(Ri,p)
    demx=0; %the deminator of the rational basis functions
    demy=0;
%     demz=0;
    for i=1:N
        demx=demx+NN(i,p+1)*Wx(i);
        demy=demy+NN(i,p+1)*Wy(i);
%         demz=demz+NN(i,p+1)*Wz(i);
    end
        
%calculating x and y coordinate of points on curve by multiplying basis 
%function in last column of NN matrix by x and y coordinate of control
%poins:
    cx=0;
    cy=0;
%     cz=0;
    for i=1:N
        cx=cx+NN(i,p+1)*Wx(i)/demx*xN(i);  %temp variable Ri,p=NN(i,p+1)*W(i)/dem
        cy=cy+NN(i,p+1)*Wy(i)/demy*yN(i);  %temp variable
%         cz=cz+NN(i,p+1)*Wz(i)/demz*zN(i);
    end
    Joint(1)=cx;     %x coordinate of points on curve for
                      %specified parameter u 
    Joint(2)=cy;     %y coordinate of points on curve for specified parameter u 
%     Cz(count)=cz;     %z coordinate of points on curve for specified parameter u 


function r=stepp(u,U)
if u>=U
    r=1;
else 
    r=0;
end
if u==1
  if u>U
    r=1;
else 
    r=0;
  end  
end
